import functools
import monilogger._monilogger as mnlg

__all__ = ('register', 'register_monilogger', 'define_event')

def register(event):
  def wrapped(func):
    if isinstance(event, str):
      event_name = event
    else:
      event_name = event.__qualname__
    @functools.wraps(func)
    def new_func(context):
      func(context)
    mnlg.register(event_name, new_func)
    new_func.stop = lambda : mnlg.stop(event_name, new_func)
    return new_func
  return wrapped

def register_monilogger(func, event):
  register(event)(func)

def define_event(event, triggering_events):
  if isinstance(event, str):
    event_name = event
  else:
    event_name = event.__qualname__
  mnlg.define_composite_event(event_name, triggering_events)



import os
import re
import subprocess
import sys
from sysconfig import get_paths

from pybind11 import get_cmake_dir
from pybind11.setup_helpers import Pybind11Extension, build_ext
from setuptools import Extension, find_packages, setup
from distutils import ccompiler

__version__ = "0.0.4"

# Convert distutils Windows platform specifiers to CMake -A arguments
PLAT_TO_CMAKE = {
    "win32": "Win32",
    "win-amd64": "x64",
    "win-arm32": "ARM",
    "win-arm64": "ARM64",
}

# A CMakeExtension needs a sourcedir instead of a file list.
# The name must be the _single_ output extension from the CMake build.
# If you need multiple extensions, see scikit-build.
class CMakeExtension(Extension):
    def __init__(self, name, sourcedir=""):
        Extension.__init__(self, name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)


class CMakeBuild(build_ext):
    def build_extension(self, ext):
        extdir = os.path.abspath(os.path.dirname(self.get_ext_fullpath(ext.name)))

        # required for auto-detection & inclusion of auxiliary "native" libs
        if not extdir.endswith(os.path.sep):
            extdir += os.path.sep

        # CMake lets you override the generator - we need to check this.
        # Can be set with Conda-Build, for example.
        cmake_generator = os.environ.get("CMAKE_GENERATOR", "")

        # Set Python_EXECUTABLE instead if you use PYBIND11_FINDPYTHON
        # EXAMPLE_VERSION_INFO shows you how to pass a value into the C++ code
        # from Python.
        cmake_args = [
            f"-DCMAKE_LIBRARY_OUTPUT_DIRECTORY={extdir}",
            f"-DPYTHON_EXECUTABLE={sys.executable}",
            f"-DCMAKE_BUILD_TYPE=Debug",  # not used on MSVC, but no harm
        ]
        build_args = []
        # Adding CMake arguments set as environment variable
        # (needed e.g. to build for ARM OSx on conda-forge)
        if "CMAKE_ARGS" in os.environ:
            cmake_args += [item for item in os.environ["CMAKE_ARGS"].split(" ") if item]

        # if self.compiler.compiler_type != "msvc":
        #     # Using Ninja-build since it a) is available as a wheel and b)
        #     # multithreads automatically. MSVC would require all variables be
        #     # exported for Ninja to pick it up, which is a little tricky to do.
        #     # Users can override the generator with CMAKE_GENERATOR in CMake
        #     # 3.15+.
        #     if not cmake_generator:
        #         try:
        #             import ninja  # noqa: F401

        #             cmake_args += ["-GNinja"]
        #         except ImportError:
        #             pass

        # else:

        #     # Single config generators are handled "normally"
        #     single_config = any(x in cmake_generator for x in {"NMake", "Ninja"})

        #     # CMake allows an arch-in-generator style for backward compatibility
        #     contains_arch = any(x in cmake_generator for x in {"ARM", "Win64"})

        #     # Specify the arch if using MSVC generator, but only if it doesn't
        #     # contain a backward-compatibility arch spec already in the
        #     # generator name.
        #     if not single_config and not contains_arch:
        #         cmake_args += ["-A", PLAT_TO_CMAKE[self.plat_name]]

        #     # Multi-config generators have a different way to specify configs
        #     if not single_config:
        #         cmake_args += [
        #             f"-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_{cfg.upper()}={extdir}"
        #         ]
        #         build_args += ["--config", cfg]

        if sys.platform.startswith("darwin"):
            # Cross-compile support for macOS - respect ARCHFLAGS if set
            archs = re.findall(r"-arch (\S+)", os.environ.get("ARCHFLAGS", ""))
            if archs:
                cmake_args += ["-DCMAKE_OSX_ARCHITECTURES={}".format(";".join(archs))]

        # Set CMAKE_BUILD_PARALLEL_LEVEL to control the parallel build level
        # across all generators.
        if "CMAKE_BUILD_PARALLEL_LEVEL" not in os.environ:
            # self.parallel is a Python 3 only way to set parallel jobs by hand
            # using -j in the build_ext call, not supported by pip or PyPA-build.
            if hasattr(self, "parallel") and self.parallel:
                # CMake 3.12+ only.
                build_args += [f"-j{self.parallel}"]

        build_temp = os.path.join(self.build_temp, ext.name)
        if not os.path.exists(build_temp):
            os.makedirs(build_temp)

        subprocess.check_call(["cmake", ext.sourcedir] + cmake_args, cwd=build_temp)
        subprocess.check_call(["cmake", "--build", "."] + build_args, cwd=build_temp)

ext_modules = [
    CMakeExtension("monilogger._monilogger", sourcedir="src/api")
]

# ext_modules = [
#     Pybind11Extension("monilogger._monilogger",
#         ["src/MoniLogger.cc"],
#         include_dirs = ["src/monilogger/include"],
#         define_macros = [('VERSION_INFO', __version__)],
#         cxx_std=17
#         ),
# ]

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="monilogger",
    version=__version__,
    author="Dorian Leroy",
    author_email="dorian.leroy@cea.fr",
    url="https://github.com/cea-hpc/monilogger",
    description="A Python logging and monitoring toolbox for C++ applications.",
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=["monilogger"],
    package_dir={"": "src/api"},
    package_data={
        "monilogger": ["include/MoniLogger.h"],
    },
    ext_modules=ext_modules,
    cmdclass={"build_ext": CMakeBuild},
    extras_require={"test": "pytest"},
    test_suite='tests',
    zip_safe=False,
    python_requires=">=3.6",
)
